@@ -113,20 +113,36 @@ def error (s):
     errorport.write (_ ("error: ") + s + '\n')
     raise _ ("Exiting ... ")
 
-def setup_temp ():
-    """
-    Create a temporary directory, and return its name.
-    """
+temp_dir = None
+class TempDirectory:
+    def __init__ (self, name=None):
+        import tempfile
+        if name:
+            if not os.path.isdir (name):
+                os.makedirs (name)
+            self.dir = name
+        else:
+            self.dir = tempfile.mkdtemp ()
+
+        os.chdir (self.dir)
+        
+    def clean (self):
+        import shutil
+        shutil.rmtree (self.dir)
+    def __del__ (self):
+        self.clean ()
+    def __call__ (self):
+        return self.dir
+    def __repr__ (self):
+        return self.dir
+    def __str__ (self):
+        return self.dir
+
+def setup_temp  (name):
     global temp_dir
-    if not options.keep_temp_dir:
-        temp_dir = tempfile.mkdtemp (program_name)
-
-    try:
-        os.mkdir (temp_dir, 0700)
-    except OSError:
-        pass
-
-    return temp_dir
+    if not temp_dir:
+        temp_dir = TempDirectory (name)
+    return temp_dir ()
 
 def popen (cmd, mode = 'r', ignore_error = 0):
     if options.verbose:
@@ -158,13 +174,6 @@ def system (cmd, ignore_error = 0):
         progress ('\n')
     return st
 
-def cleanup_temp ():
-    if not options.keep_temp_dir:
-        if options.verbose:
-            progress (_ ("Cleaning %s...") % temp_dir)
-        shutil.rmtree (temp_dir)
-
-
 def strip_extension (f, ext):
     (p, e) = os.path.splitext (f)
     if e == ext:
@@ -181,7 +190,7 @@ options = None
 exit_value = 0
 backend_options = ''
 program_name = 'mftrace'
-temp_dir = os.path.join (os.getcwd (), program_name + '.dir')
+temp_dir = None
 program_version = '@VERSION@'
 origdir = os.getcwd ()
 
@@ -211,7 +220,7 @@ def find_file (nm):
         p = os.path.join (d, nm)
         try:
             open (p)
-            return p
+            return os.path.abspath (p)
         except IOError:
             pass
 
@@ -518,7 +527,12 @@ def trace_font (fontname, gf_file, metric, glyphs, encoding,
     t1os = []
     font_bbox = (10000, 10000, -10000, -10000)
 
-    progress (_ ("Tracing bitmaps... "))
+    progress (_ ("Tracing bitmaps..."))
+
+    if options.verbose:
+        progress ('\n')
+    else:
+        progress (' ')
 
     # for single glyph testing.
     # glyphs = []
@@ -565,7 +579,8 @@ def trace_font (fontname, gf_file, metric, glyphs, encoding,
 
         t1os.append ('\n/%s %s ' % (encoding[a], t1o))
 
-    progress ('\n')
+    if not options.verbose:
+        progress ('\n')
     to_type1 (t1os, font_bbox, fontname, encoding, magnification, fontinfo)
 
 def ps_encode_encoding (encoding):
@@ -797,6 +812,8 @@ def assemble_font (fontname, format, is_raw):
     outname = fontname + ext
 
     progress (_ ("Assembling raw font to `%s'... ") % outname)
+    if options.verbose:
+        progress ('\n')
     system ('t1asm %s mftrace.t1asm %s' % (asm_opt, shell_escape_filename (outname)))
     progress ('\n')
     return outname
@@ -881,33 +898,14 @@ def gen_pixel_font (filename, metric, magnification):
     try:
         open ('%s.%dgf' % (filename, prod))
     except IOError:
-        os.environ['KPSE_DOT'] = '%s:' % origdir
 
+        ## MFINPUTS/TFMFONTS take kpathsea specific values;
+        ## we should analyse them any further.
         os.environ['MFINPUTS'] = '%s:%s' % (origdir,
                           getenv ('MFINPUTS', ''))
         os.environ['TFMFONTS'] = '%s:%s' % (origdir,
                           getenv ('TFMINPUTS', ''))
 
-        # FIXME: we should not change to another (tmp) dir?
-        # or else make all relavitive dirs in paths absolute.
-        def abs_dir (x, dir):
-            if x and os.path.abspath (x) != x:
-                return os.path.join (dir, x)
-            return x
-
-        def abs_path (path, dir):
-            # Python's ABSPATH means ABSDIR
-            dir = os.path.abspath (dir)
-            return string.join (map (lambda x: abs_dir (x, dir),
-                        string.split (path,
-                               os.pathsep)),
-                      os.pathsep)
-
-        os.environ['MFINPUTS'] = abs_path (os.environ['MFINPUTS'],
-                         origdir)
-        os.environ['TFMFONTS'] = abs_path (os.environ['TFMFONTS'],
-                         origdir)
-
         progress (_ ("Running Metafont..."))
 
         cmdstr = r"mf '\mode:=lexmarks; mag:=%f; nonstopmode; input %s'" %  (mag, filename)
@@ -1320,24 +1318,49 @@ def do_file (filename):
     progress (_ ("Font `%s'..." % basename))
     progress ('\n')
 
+    ## setup encoding
+    if encoding_file and not os.path.exists (encoding_file):
+        encoding_file = find_file (encoding_file)
+    elif encoding_file:
+        encoding_file = os.path.abspath (encoding_file)
+
+    ## setup TFM
+    if options.tfm_file:
+        options.tfm_file = os.path.abspath (options.tfm_file)
+    else:
+        tfm_try = find_file (basename + '.tfm')
+        if tfm_try:
+            options.tfm_file = tfm_try
+
+    if not os.environ.has_key ("MFINPUTS"):
+         os.environ["MFINPUTS"] = os.getcwd () + ":"
+
+    ## must change dir before calling mktextfm.
+    if options.keep_temp_dir:
+        def nop():
+            pass
+        setup_temp (os.path.join (os.getcwd (), program_name + '.dir'))
+        temp_dir.clean = nop
+    else:
+        setup_temp (None)
+        
+    if options.verbose:
+        progress ('Temporary directory is `%s\'\n' % temp_dir)
+    
     if not options.tfm_file:
         options.tfm_file = popen ("mktextfm %s 2>/dev/null" % shell_escape_filename (basename)).read ()
         if options.tfm_file:
-            options.tfm_file = options.tfm_file[:-1]
+            options.tfm_file = options.tfm_file.strip ()
+            options.tfm_file = os.path.abspath (options.tfm_file)
 
     if not options.tfm_file:
         error (_ ("Can not find a TFM file to match `%s'") % basename)
 
-    options.tfm_file = os.path.abspath (options.tfm_file)
     metric = tfm.read_tfm_file (options.tfm_file)
 
     fontinfo = guess_fontinfo (basename)
     fontinfo.update (options.font_info)
 
-    if encoding_file and not os.path.exists (encoding_file):
-        encoding_file = find_file (encoding_file)
-
-
     if not encoding_file:
         codingfile = 'tex256.enc'
         if not coding_dict.has_key (metric.coding):
@@ -1354,14 +1377,6 @@ def do_file (filename):
     if not len (options.glyphs):
         options.glyphs = range (0, len (encoding))
 
-    global temp_dir
-    temp_dir = setup_temp ()
-
-    if options.verbose:
-        progress ('Temporary directory is `%s\'\n' % temp_dir)
-
-    os.chdir (temp_dir)
-
     if not options.gffile:
         # run mf
         base = gen_pixel_font (basename, metric, options.magnification)
@@ -1378,7 +1393,6 @@ def do_file (filename):
         shutil.copy2 (basename + '.' + format, origdir)
 
     os.chdir (origdir)
-    cleanup_temp ()
 
 
 
